@@ -24,6 +24,7 @@ from sqlalchemy import types as sa_types
 from sqlalchemy import schema as sa_schema
 from sqlalchemy import util
 from sqlalchemy.sql import compiler
+from sqlalchemy.sql import operators
 from sqlalchemy.engine import default
 from sqlalchemy import __version__ as SA_Version
 from . import reflection as ibm_reflection
@@ -104,7 +105,7 @@ RESERVED_WORDS = set(
     'unnest', 'element', 'percentile_disc', 'upper', 'exec', 'power', 'var_pop', 'exp',
     'real', 'var_samp', 'false', 'recursive', 'varchar', 'filter', 'ref', 'varying',
     'float', 'regr_avgx', 'width_bucket', 'floor', 'regr_avgy', 'window', 'fusion',
-    'regr_count', 'within'])
+    'regr_count', 'within', 'asc'])
 
 
 class _IBM_Boolean(sa_types.Boolean):
@@ -317,6 +318,13 @@ class DB2TypeCompiler(compiler.GenericTypeCompiler):
 
 class DB2Compiler(compiler.SQLCompiler):
 
+    if SA_Version < [0, 9]:
+        def visit_false(self, expr, **kw):
+            return '0'
+            
+        def visit_true(self, expr, **kw):
+            return '1'
+            
     def visit_now_func(self, fn, **kw):
         return "CURRENT_TIMESTAMP"
 
@@ -456,6 +464,14 @@ class DB2Compiler(compiler.SQLCompiler):
 
     def visit_release_savepoint(self, savepoint_stmt):
         return 'RELEASE TO SAVEPOINT %(sid)s'% {'sid':self.preparer.format_savepoint(savepoint_stmt)}
+    
+    def visit_unary(self, unary, **kw):
+        if (unary.operator == operators.exists)  and kw.get('within_columns_clause', False):
+            usql = super(DB2Compiler, self).visit_unary(unary, **kw)
+            usql = "CASE WHEN " + usql + " THEN 1 ELSE 0 END"
+            return usql
+        else:
+            return super(DB2Compiler, self).visit_unary(unary, **kw)
 
 class DB2DDLCompiler(compiler.DDLCompiler):
 
@@ -515,16 +531,16 @@ class DB2DDLCompiler(compiler.DDLCompiler):
         if (self.dialect.dbms_name.find('DB2/') != -1) and ([long(ver_token) for ver_token in self.dialect.dbms_ver.split('.')[0:2]] >= [10, 5]):
             for constraint in table._sorted_constraints:
                 if isinstance(constraint, sa_schema.UniqueConstraint):
-                    for column in constraint.columns._all_cols:
+                    for column in constraint:
                         if column.nullable:
                             constraint.use_alter = True
                             break
                     if hasattr(constraint, 'use_alter') and constraint.use_alter:
                         if not constraint.name:
-                            index_name = "%s_%s_%s" % ('ukey', self.preparer.format_table(constraint.table), '_'.join(column.name for column in constraint.columns._all_cols))
+                            index_name = "%s_%s_%s" % ('ukey', self.preparer.format_table(constraint.table), '_'.join(column.name for column in constraint))
                         else:
                             index_name = constraint.name
-                        index = sa_schema.Index(index_name, *(column for column in constraint.columns._all_cols))
+                        index = sa_schema.Index(index_name, *(column for column in constraint))
                         index.unique = True
                         index.exclude_nulls = True
         result = super( DB2DDLCompiler, self ).create_table_constraints(table, **kw)
@@ -542,16 +558,16 @@ class DB2DDLCompiler(compiler.DDLCompiler):
     def visit_add_constraint(self, create):
         if (self.dialect.dbms_name.find('DB2/') != -1) and ([long(ver_token) for ver_token in self.dialect.dbms_ver.split('.')[0:2]] >= [10, 5]):
             if isinstance(create.element, sa_schema.UniqueConstraint):
-                for column in create.element.columns._all_cols:
+                for column in create.element:
                     if column.nullable:
                         create.element.exclude_nulls = True
                         break
                 if getattr(create.element, 'exclude_nulls', None):
                     if not create.element.name:
-                        index_name = "%s_%s_%s" % ('uk_index', self.preparer.format_table(create.element.table), '_'.join(column.name for column in create.element.columns._all_cols))
+                        index_name = "%s_%s_%s" % ('uk_index', self.preparer.format_table(create.element.table), '_'.join(column.name for column in create.element))
                     else:
                         index_name = create.element.name
-                    index = sa_schema.Index(index_name, *(column for column in create.element.columns._all_cols))
+                    index = sa_schema.Index(index_name, *(column for column in create.element))
                     index.unique = True
                     index.exclude_nulls = True
                     sql = self.visit_create_index(sa_schema.CreateIndex(index)) 
@@ -616,6 +632,7 @@ class DB2Dialect(default.DefaultDialect):
     supports_sane_rowcount = True
     supports_sane_multi_rowcount = True
     supports_native_decimal = False
+    supports_native_boolean = False
     preexecute_sequences = False
     supports_alter = True
     supports_sequences = True
@@ -690,10 +707,18 @@ class DB2Dialect(default.DefaultDialect):
     def get_foreign_keys(self, connection, table_name, schema=None, **kw):
         return self._reflector.get_foreign_keys(
                                 connection, table_name, schema=schema, **kw)
+        
+    def get_incoming_foreign_keys(self, connection, table_name, schema=None, **kw):
+        return self._reflector.get_incoming_foreign_keys(
+                                connection, table_name, schema=schema, **kw)
 
     def get_indexes(self, connection, table_name, schema=None, **kw):
         return self._reflector.get_indexes(
                                 connection, table_name, schema=schema, **kw)
+        
+    def get_unique_constraints(self, connection, table_name, schema=None, **kw):
+        return self._reflector.get_unique_constraints(
+                                connection, table_name, schema=schema, **kw)
 
 
 # legacy naming
