@@ -101,6 +101,21 @@ class DB2Reflector(BaseReflector):
       Column("INDNAME", CoerceUnicode, key="indname"),
       Column("COLNAMES", CoerceUnicode, key="colnames"),
       Column("UNIQUERULE", CoerceUnicode, key="uniquerule"),
+      Column("SYSTEM_REQUIRED", CoerceUnicode, key="system_required"),
+      schema="SYSCAT")
+    
+    sys_tabconst = Table("TABCONST", ischema,
+      Column("TABSCHEMA", CoerceUnicode, key="tabschema"),
+      Column("TABNAME", CoerceUnicode, key="tabname"),
+      Column("CONSTNAME", CoerceUnicode, key="constname"),
+      Column("TYPE", CoerceUnicode, key="type"),
+      schema="SYSCAT")
+    
+    sys_keycoluse = Table("KEYCOLUSE", ischema,
+      Column("TABSCHEMA", CoerceUnicode, key="tabschema"),
+      Column("TABNAME", CoerceUnicode, key="tabname"),
+      Column("CONSTNAME", CoerceUnicode, key="constname"),
+      Column("COLNAME", CoerceUnicode, key="colname"),
       schema="SYSCAT")
 
     sys_foreignkeys = Table("SQLFOREIGNKEYS", ischema,
@@ -305,6 +320,48 @@ class DB2Reflector(BaseReflector):
                 fschema[r[0]]['constrained_columns'].append(self.normalize_name(r[3]))
                 fschema[r[0]]['referred_columns'].append(self.normalize_name(r[7]))
         return [value for key, value in fschema.iteritems()]
+    
+    @reflection.cache
+    def get_incoming_foreign_keys(self, connection, table_name, schema=None, **kw):
+        default_schema = self.default_schema_name
+        current_schema = self.denormalize_name(schema or default_schema)
+        default_schema = self.normalize_name(default_schema)
+        table_name = self.denormalize_name(table_name)
+        sysfkeys = self.sys_foreignkeys
+        query = sql.select([sysfkeys.c.fkname, sysfkeys.c.fktabschema, \
+                            sysfkeys.c.fktabname, sysfkeys.c.fkcolname, \
+                            sysfkeys.c.pkname, sysfkeys.c.pktabschema, \
+                            sysfkeys.c.pktabname, sysfkeys.c.pkcolname],
+            sql.and_(
+              sysfkeys.c.pktabschema == current_schema,
+              sysfkeys.c.pktabname == table_name
+            ),
+            order_by=[sysfkeys.c.colno]
+          )
+
+        fschema = {}
+        for r in connection.execute(query):
+            if not fschema.has_key(r[0]):
+                constrained_schema = self.normalize_name(r[1])
+
+                # if no schema specified and referred schema here is the
+                # default, then set to None
+                if schema is None and \
+                    constrained_schema == default_schema:
+                    constrained_schema = None
+
+                fschema[r[0]] = {
+                    'name': self.normalize_name(r[0]),
+                    'constrained_schema': constrained_schema,
+                    'constrained_table': self.normalize_name(r[2]),
+                    'constrained_columns': [self.normalize_name(r[3])],
+                    'referred_schema': schema,
+                    'referred_table': self.normalize_name(r[6]),
+                    'referred_columns': [self.normalize_name(r[7])]}
+            else:
+                fschema[r[0]]['constrained_columns'].append(self.normalize_name(r[3]))
+                fschema[r[0]]['referred_columns'].append(self.normalize_name(r[7]))
+        return [value for key, value in fschema.iteritems()]
 
 
     @reflection.cache
@@ -312,7 +369,7 @@ class DB2Reflector(BaseReflector):
         current_schema = self.denormalize_name(schema or self.default_schema_name)
         table_name = self.denormalize_name(table_name)
         sysidx = self.sys_indexes
-        query = sql.select([sysidx.c.indname, sysidx.c.colnames, sysidx.c.uniquerule],
+        query = sql.select([sysidx.c.indname, sysidx.c.colnames, sysidx.c.uniquerule, sysidx.c.system_required],
             sql.and_(
               sysidx.c.tabschema == current_schema,
               sysidx.c.tabname == table_name
@@ -323,6 +380,8 @@ class DB2Reflector(BaseReflector):
         col_finder = re.compile("(\w+)")
         for r in connection.execute(query):
             if r[2] != 'P':
+                if r[2] == 'U' and r[3] != 0:
+                    continue
                 indexes.append({
                         'name': self.normalize_name(r[0]),
                         'column_names': [self.normalize_name(col)
@@ -330,6 +389,34 @@ class DB2Reflector(BaseReflector):
                         'unique': r[2] == 'U'
                     })
         return indexes
+    
+    @reflection.cache
+    def get_unique_constraints(self, connection, table_name, schema=None, **kw):
+        current_schema = self.denormalize_name(schema or self.default_schema_name)
+        table_name = self.denormalize_name(table_name)
+        syskeycol = self.sys_keycoluse
+        sysconst = self.sys_tabconst
+        query = sql.select([syskeycol.c.constname, syskeycol.c.colname],
+            sql.and_(
+              syskeycol.c.constname == sysconst.c.constname,
+              sysconst.c.tabname == table_name,
+              sysconst.c.tabschema == current_schema,
+              sysconst.c.type == 'U'
+            ),
+            order_by=[syskeycol.c.constname]
+          )
+        uniqueConsts = []
+        currConst = None
+        for r in connection.execute(query):
+            if currConst == r[0]:
+                uniqueConsts[-1]['column_names'].append(self.normalize_name(r[1]))
+            else:
+                currConst = r[0]
+                uniqueConsts.append({
+                        'name': self.normalize_name(currConst),
+                        'column_names': [self.normalize_name(r[1])],
+                    })
+        return uniqueConsts
 
 class AS400Reflector(BaseReflector):
 
